// Copyright (c) 2012 Cloudera, Inc.  All rights reserved.
define([
	"cloudera/cmf/include/RepositoryValidator", 
	"cloudera/cmf/include/CDHSelectionPage"
], function(RepositoryValidator, CDHSelectionPage) {

	/**
	 * Controls the Add Hosts SSH page and the CDH Repository selection page.
	 * This module uses KnockoutJS to bind the state of the DOM to the
	 * state of the view model. Either the first step is selected (the SSH page
	 * will be displayed), or the second step is selected (the Repository selection page
	 * will be displayed).
	 */
	return function(options) {
		var self = this;
		$("#mainForm").validate({
			rules: {
				cdhCustomUrl: "repository",
				impalaCustomUrl: "repository",
				cmCustomUrl: "repository"
			}
		});

		/**
		 * Initialize the knockout state. Note: we may
		 * be entering the page via the back button.
		 * In which case, we want to initialize the state
		 * based on the DOM value.
		 */
		function initParams() {
			self.chosenStep = ko.observable(-1);

			// various things for the SSH page.
			// "true" if root selected.
			var isRoot = $("input[name=isRoot]:checked").val() || "true";
			self.isRoot = ko.observable(isRoot);

			// the other username.
			var username = $("input[name=username][type=text]").val() || "";
			self.username = ko.observable(username);

			// Authentication method, whether to use password or passphrase.
			var usePassword = $("input[name=usePassword]:checked").val() || "true";
			self.usePassword = ko.observable(usePassword);

			// Password & confirmation.
			var password = $("input[name=password]").val() || "";
			self.password = ko.observable(password);

			var passwordConfirm = $("input[name=passwordConfirm]").val() || "";
			self.passwordConfirm = ko.observable(passwordConfirm);

			// Passphrase & confirmation.
			var passphrase = $("input[name=passphrase]").val() || "";
			self.passphrase = ko.observable(passphrase);

			var passphraseConfirm = $("input[name=passphraseConfirm]").val() || "";
			self.passphraseConfirm = ko.observable(passphraseConfirm);

			// No need to show any errors initially.
			self.usernameError = ko.observable("");
			self.passwordError = ko.observable("");
			self.passphraseError = ko.observable("");
			self.privateKeyError = ko.observable("");

			// Observables for the CDH selection page and knockout helper methods
			// are initialized in this call.
			CDHSelectionPage.initializeKnockout(self);
		
			// Enable the submit button by default.
			self.submitted = ko.observable(false);
		}

		initParams();

		/**
		 * Returns true if the passwords match.
		 */
		self.passwordMatch = ko.dependentObservable(function () {
			return self.usePassword() !== 'true' || self.password() === self.passwordConfirm();
		}, self);

		/**
		 * Returns true if the passphrases match.
		 */
		self.passphraseMatch = ko.dependentObservable(function () {
			return self.usePassword() === 'true' || self.passphrase() === self.passphraseConfirm();
		}, self);

		/**
		 * Returns true if there is a user.
		 */
		self.userEntered = ko.dependentObservable(function () {
			return self.isRoot() === 'true' || self.username() !== '';
		}, self);

		/**
		 * Allow one click on the start installation button.
		 */
		self.enableContinueForSSH = ko.dependentObservable(function () {
			return !self.submitted();
		}, self);

		function focusAndSelect($el, showError){
			if (showError) {
				$el.closest(".control-group").addClass("error");
			}
			$el.focus();
			$el.select();
		}

		function resetErrors () {
			$(".control-group").removeClass("error");
			self.passwordError("");
			self.passphraseError("");
			self.usernameError("");
			self.privateKeyError("");
		}

		function checkUsername () {
			if (!self.userEntered()) {
				self.usernameError(options.messages.usernameNotEntered);
				focusAndSelect($("input[name=username]"), true);
				return false;
			}
			return true;
		}

		function checkPasswordMatch () {
			if (!self.passwordMatch()) {
				self.passwordError(options.messages.passwordDontMatch);
				focusAndSelect($("input[name=passwordConfirm]"), true);
				focusAndSelect($("input[name=password]"), true);
				return false;
			}
			return true;
		}

		function checkPassphraseMatch () {
			if (!self.passphraseMatch()) {
				self.passphraseError(options.messages.passphraseDontMatch);
				focusAndSelect($("input[name=passphraseConfirm]"), true);
				focusAndSelect($("input[name=passphrase]"), true);
				return false;
			}
			return true;
		}

		function checkEmptyPassword () {
			if (self.password() === '' && self.passwordConfirm() === '') {
				if (!confirm(options.messages.continueWithoutPasswordPrompt)) {
					focusAndSelect($("input[name=password]"), false);
					return false;
				}
			}
			return true;
		}

		function checkEmptyPassphrase () {
			if (self.passphrase() === '' && self.passphraseConfirm() === '') {
				if (!confirm(options.messages.continueWithoutPassphrasePrompt)) {
					focusAndSelect($("input[name=passphrase]"), false);
					return false;
				}
			}
			return true;
		}

		function checkEmptyPrivateKey() {
			var result = true;
			if ($("[name=privateKey]").val() === '') {
				self.privateKeyError(options.messages.missingPrivateKey);
				focusAndSelect($("input[name=privateKey]"), true);
				result = false;
			}
			return result;
		}

		self.checkFormValid = function () {
			return $("#mainForm").valid();
		};

		/**
		 * When the Continue button is clicked. Going from the CDH Version page
		 * to the SSH Version page. Need to validate all the CDH related fields.
		 */
		self.continueCDHClicked = function (vm, event) {
			var result = self.checkCDHPage(vm, event);
			if (result) {
				location.hash = "ssh";
			}
			event.preventDefault();
		};

		self.continueSSHClicked = function (vm, event) {
			var result = self.checkSSHPage(vm, event);
			if (result) {
				self.submitted(true);
			} else {
				event.preventDefault();
			}
		};

		self.backClicked = function (vm, event) {
			history.go(-1);
		};

		self.checkSSHPage = function (vm, event) {
			resetErrors();
			if (!self.checkFormValid()) {
				return false;
			}
			var result = checkUsername();

			// checkXXXMatch is always executed because
			// we want to show all the errors.
			// checkEmptyXXX is only executed if
			// everything else passes.
			if (self.usePassword() === 'true') {
				result = checkPasswordMatch() && result;
				result = result && checkEmptyPassword();
			} else {
				result = checkPassphraseMatch() && result;
				result = checkEmptyPrivateKey() && result;
				result = result && checkEmptyPassphrase();
			}
			return result;
		};

		self.checkCDHPage = function (vm, event) {
			resetErrors();
			return self.checkFormValid();
		};

		/**
		 * When the Start Installation button is clicked.
		 */
		$('#mainForm').submit(function (event) {
			if (self.chosenStep() === 0) {
				self.continueCDHClicked(self, event);
			} else {
				self.continueSSHClicked(self, event);
			}
		});

		self.updatePageTitle = function (title) {
			setTimeout(function() {
				$.publish("updatePageTitle", [title]);
			}, 200);
		};

		$.sammy(function () {
			this.get("ssh#ssh", function () {
				self.chosenStep(1);
				self.updatePageTitle($("#sshTitle").text());
			});

			this.get("ssh", function () {
				self.chosenStep(0);
				self.updatePageTitle($("#reposTitle").text());
			});
		}).run();
	};
});
